@@ -8,7 +8,7 @@ use WWW::Wikipedia::Entry;
 
 use base qw( LWP::UserAgent );
 
-our $VERSION = '1.99';
+our $VERSION = '2.00';
 
 use constant WIKIPEDIA_URL =>
     'http://%s.wikipedia.org/w/index.php?title=%s&action=raw';
@@ -148,19 +148,19 @@ sub search {
 
     croak( "search() requires you pass in a string" ) if !defined( $string );
     
-    $string = utf8::is_utf8( $string )
+    my $enc_string = utf8::is_utf8( $string )
         ? URI::Escape::uri_escape_utf8( $string )
         : URI::Escape::uri_escape( $string );
-    my $src = sprintf( WIKIPEDIA_URL, $self->language(), $string );
+    my $src = sprintf( WIKIPEDIA_URL, $self->language(), $enc_string );
 
     my $response = $self->get( $src );
     if ( $response->is_success() ) {
-        my $entry = WWW::Wikipedia::Entry->new( $response->content(), $src );
+        my $entry = WWW::Wikipedia::Entry->new( $response->decoded_content(), $src );
 
         # look for a wikipedia style redirect and process if necessary
+        # try to catch self-redirects
         return $self->search( $1 )
-            if $self->follow_redirects
-                && $entry->text() =~ /^#REDIRECT ([^\r\n]+)/is;
+            if $self->follow_redirects && $entry->raw() =~ /^#REDIRECT \[\[([^|\]]+)/is && $1 ne $string;
 
         return ( $entry );
     }
@@ -187,7 +187,7 @@ sub random {
         my( $title ) = $response->request->uri =~ m{\.org/wiki/(.+)$};
         $src      = sprintf( WIKIPEDIA_URL, $self->language(), $title );
         $response = $self->get( $src );
-        return WWW::Wikipedia::Entry->new( $response->content(), $src );
+        return WWW::Wikipedia::Entry->new( $response->decoded_content(), $src );
     }
 
     $self->error( "uhoh, WWW::Wikipedia unable to contact " . $src );
